#ifndef SEAFILE_CLIENT_SEAFILE_ERROR_H
#define SEAFILE_CLIENT_SEAFILE_ERROR_H

#include <QObject>
#include <QString>

QString
translateSyncErrorCode(const int error_code);

QString
translateFileSyncErrorCode(const int error_code);

#endif //SEAFILE_CLIENT_SEAFILE_ERROR_H
